#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()

from code.classes.Counter import Counter
import code.functions.loading as loading


class AnimatedSprite( pygame.sprite.Sprite ):
	def __init__( self, images, transparency_colorkey, interval,
		interval_stepping, interval_repeat=True, alternate=False ):
		
		pygame.sprite.Sprite.__init__( self )

		self.images = [ loading.load_image( x ) for x in images ]
		[ x.set_colorkey( transparency_colorkey ) for x in self.images ]
			
		self.interval = interval
		self.counter = Counter( interval, interval_stepping, interval_repeat )

		# Set the first image.
		self.image_index = 0
		self.image = self.images[self.image_index]

		self.image_limit = len( self.images ) - 1
		

		
	def update( self ):

		if self.counter.update():
			# Show the next image.
			if self.image_index == self.image_limit:
				self.image_index = 0
			else:
				self.image_index += 1

			self.image = self.images[self.image_index]	
