#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()
import math

import code.functions.loading as loading
import code.constants.constants as constants


class Bit( pygame.sprite.Sprite ):
	def __init__( self, filename, center_xy_position, transparency_colorkey,
		speed, gravity_amount, angle, window_dimensions, time_addition ):
		
		pygame.sprite.Sprite.__init__( self )

		self.image = loading.load_image( filename )
		self.rect = self.image.get_rect( center = center_xy_position )
		self.image.set_colorkey( transparency_colorkey )
		self.transparency_colorkey = transparency_colorkey
		self.speed = speed
		self.gravity_amount = gravity_amount
		self.angle = angle
		self.visible_area = pygame.Rect( *window_dimensions )

		self.timer = 0
		self.time_addition = time_addition

		self.speedy = math.sin( math.radians( self.angle ) ) * self.speed


	def update( self ):
		speedx, speedy = self.calculate_xy_speed()
		self.rect.move_ip( speedx, speedy )

		if not self.visible_area.contains( self.rect ):
			self.kill()


	def calculate_xy_speed( self ):
		self.timer += self.time_addition
		speedx = math.cos( math.radians( self.angle ) ) * self.speed
		speedy = math.sin( math.radians( self.angle ) ) * self.speed + ( constants.GFORCE * self.timer )

		return speedx, speedy
		
