#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()

import code.functions.loading as loading
from code.classes.BitWalker import BitWalker


class EightBit( pygame.sprite.Sprite ):	

	def __init__( self, filename, position_xy_center, bit_walker_data,
		image_bit_on, image_bit_off, image_no_bit, transparency_colorkey,
		bit_drawing_center_xy_positions, byte_assemblies ):
		pygame.sprite.Sprite.__init__( self )

		self.image = loading.load_image( filename )
		self.rect = self.image.get_rect( center = position_xy_center )

		self.bitwalker = pygame.sprite.GroupSingle( BitWalker( **bit_walker_data ) )

		self.byte = list( "00000000" )
		self.bits_to_draw = pygame.sprite.Group()

		self.image_bit_one = loading.load_image( image_bit_on )
		self.image_bit_one.set_colorkey( transparency_colorkey )
		self.image_bit_null = loading.load_image( image_bit_off )
		self.image_bit_null.set_colorkey( transparency_colorkey )		
		self.image_no_bit = loading.load_image( image_no_bit )
		self.image_no_bit.set_colorkey( transparency_colorkey )		

		self.bit_drawing_center_xy_positions = bit_drawing_center_xy_positions
		self.byte_assemblies = byte_assemblies

		for x in range( 8 ):
			bit_sprite = pygame.sprite.Sprite()
			bit_sprite.index = x
			bit_sprite.image = self.image_bit_null
			bit_sprite.rect = bit_sprite.image.get_rect( center= self.bit_drawing_center_xy_positions[x] )
			self.bits_to_draw.add( bit_sprite )
				


	def update( self ):

		self.bitwalker.update()
		self.display_byte()


	def set_bit( self, bit_status ):

		if bit_status == True:
			# Set a '1' bit.
			self.byte[self.bitwalker.sprite.chosen_bit] = '1'
			
		else:
			# Set a '0' bit.
			self.byte[self.bitwalker.sprite.chosen_bit] = '0'


	def display_byte( self ):
	
		for x in range( len( self.byte ) ):
			if self.byte[x] == '1':
				for y in self.bits_to_draw:
					if y.index == x:
						y.image = self.image_bit_one
			if self.byte[x] == '0':			
				for y in self.bits_to_draw:
					if y.index == x:
						y.image = self.image_bit_null


	def check_byte( self, player_ship ):
		for x in self.byte_assemblies.items():
			if x[0] == ''.join( self.byte ):
				getattr( player_ship, x[1][0] )( x[1][1] )
				self.byte = list( "00000000" )
