#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import copy

import pygame
pygame.init()

from code.classes.AnimatedSprite import AnimatedSprite
from code.classes.Projectile import Projectile

import random
import math


class EnemySpaceship( AnimatedSprite ):
	def __init__( self, images, center_xy_position, transparency_colorkey,
		interval, interval_stepping, removal_area, speed, action, delay,
		projectile_data ):
		
		super( EnemySpaceship, self ).__init__( images,
			transparency_colorkey, interval, interval_stepping )

		self.rect = self.image.get_rect( center=center_xy_position )
		self.projectiles = pygame.sprite.Group()
		self.image.set_alpha( 200 )

		self.transparency_colorkey = transparency_colorkey

		self.removal_area = removal_area
		self.speed = speed
		self.action = action

		self.speed_y = 0
		self.speed_x = 0

		self.time = 0
		self.delay = delay

		self.center = [ self.rect.centerx, self.rect.centery ]

		self.projectile_data = projectile_data


	def update( self, player, enemy_projectiles, wave_number ):

		if self.delay > 0:
			self.delay -= 1
			return
		
		if self.removal_area.contains( self.rect ):
			self.kill()


		x, y = getattr( self, self.action )()
		self.center[0] += x
		self.center[1] += y

		self.rect.center = self.center

		self.time += 1

		self.speed_x = x
		self.speed_y = y


		# Decide whether a shot should be fired.
		# Will later be changed to increase the likelyhood of a projectile to be fired in upper waves.
		if random.randint( 1, 1000 ) in range( 1, wave_number ) and player.sprite.health > 0:
			self.fire( player, enemy_projectiles )


	def weave( self ):

		x = 4 * math.cos( self.time * 0.5 * math.radians( math.pi ) )
		y = self.speed / 5

		return x, y


	def brake( self ):

		if self.speed > 0:
			self.speed -= 0.01

		return 0, self.speed


	def slant_left( self ):
	
		x = ( self.speed / 5 ) * -1
		y = self.speed

		return x, y


	def slant_right( self ):
	
		x = ( self.speed / 5 )
		y = self.speed

		return x, y


	def circular( self ):
		x = self.speed * math.cos( math.radians( self.time ) )
		y = self.speed * math.sin( math.radians( self.time ) ) + self.speed

		return x, y


	def normal( self ):
		x = 0
		y = self.speed

		return x, y


	def fire( self, player, enemy_projectiles ):

		player_rect = player.sprite.rect

		# Calculate the angle.
		x = player_rect.centerx - self.rect.centerx
		y = player_rect.centery - self.rect.centery

		angle = math.degrees( math.atan2( y, x ) )
		
		# Fire.
		p_data = copy.deepcopy( self.projectile_data )
		p_data["center_xy_position"] = self.rect.center
		p_data["angle"] = angle
		p_data["speed"] = random.randint( 4, 10 )
		
		enemy_projectiles.add( Projectile( **p_data ) )		
