#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()

import math

import code.functions.loading as loading
from code.classes.Counter import Counter


class Projectile( pygame.sprite.Sprite ):
	def __init__( self, filename, transparency_colorkey, center_xy_position,
		angle, speed, visible_area, interval, interval_stepping ):
		pygame.sprite.Sprite.__init__( self )

		self.image = loading.load_image( filename )
		self.image.set_colorkey( transparency_colorkey )
		self.image = pygame.transform.rotate( self.image, 360 - angle )

		self.counter = Counter( interval, interval_stepping )
		
		self.rect = self.image.get_rect( center=center_xy_position )
		self.visible_area = pygame.Rect( *visible_area )


		self.speed_x = math.cos( math.radians( angle ) ) * speed
		self.speed_y = math.sin( math.radians( angle ) ) * speed


	def update( self ):
		if self.counter.update():
			if self.visible_area.contains( self.rect ):
				self.rect.move_ip( self.speed_x, self.speed_y )
			else:
				self.kill()
