#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()

import random

import code.functions.loading as loading
from code.classes.Counter import Counter


class Star( pygame.sprite.Sprite ):

	image = None
	
	def __init__( self, filename, vertical_speed,
		relocation_line, recreation_area, creation_area, interval,
		interval_stepping, interval_repeat=True ):
		
		pygame.sprite.Sprite.__init__( self )

		if Star.image == None:
			Star.image = loading.load_image( filename )
		
		self.image = Star.image
		self.rect = self.image.get_rect()
		self.relocate( creation_area )
		
		self.vertical_speed = vertical_speed
		self.relocation_line = relocation_line
		self.recreation_area = recreation_area

		self.counter = Counter( interval, interval_stepping, interval_repeat )


	def update( self ):
		if self.counter.update():
			self.rect.move_ip( 0, self.vertical_speed )

			if self.rect.bottom >= self.relocation_line:
				self.relocate( self.recreation_area )


	def relocate( self, area ):
		self.rect.center = (
			random.randint( area[0], area[2] ),
			random.randint( area[1], area[3] )
		)
