#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import os.path

import pygame
pygame.init()


GFX_PATH = os.path.join( os.getcwd(), "gfx" )
SFX_PATH = os.path.join( os.getcwd(), "sfx" )
MUSIC_PATH = os.path.join( os.getcwd(), "music" )

GAME_TITLE = "Deep Space"

# Colours.
BLACK = pygame.Color( 0, 0, 0, 0 )
WHITE = pygame.Color( 255, 255, 255, 0 )
YELLOW = pygame.Color( 255, 255, 0, 0 )

# Frame-rate.
FRAMERATE = 50

# Window dimensions.
WINDOW_DIMENSIONS = ( 640, 480 )

# Player.
PLAYER1 = {
	"center_xy_position": ( 320, 440 ),
	"transparency_colorkey": WHITE,
	"images": ( "spaceship_player_1.png", "spaceship_player_1.png" ),
	"interval": 5,
	"interval_stepping": 1,
	"controls": {
		"move_left": [ pygame.K_LEFT, False ],
		"move_right": [ pygame.K_RIGHT, False ],
		"move_up": [ pygame.K_UP, False ],
		"move_down": [ pygame.K_DOWN, False ],
		"fire": [ pygame.K_RCTRL, False ]
	},
	"movement_speed": 8,
	"projectile_data": {
		"filename": "projectile_player_1.png",
		"transparency_colorkey": WHITE,
		"speed": 15,
		"center_xy_position": [ 0, 0 ], # To be set dynamically.
		"angle": 0, # To be set dynamically.
		"visible_area": ( 0, 0, 640, 480 ),
		"interval": 1,
		"interval_stepping": 1
	},
	"projectile_levels": {
		1: ( 270, ),
		2: ( 260, 280 ),
		3: ( 250, 270, 290 ),
		4: ( 90, 250, 270, 290 )
	},
	"current_projectile_level": 1,
	"health": 100,
	"health_topleft_coords": ( 40, 462 ),
	"score": 0,
	"score_topright_coords": ( 30, 462 ),
	"dead": False,
	"movement_boundaries": pygame.Rect( 0, 0, 640, 480 ),
	"projectile_topleft_coords": ( 50, 462 )
}


# Star.
STAR = {
	"filename": "star.png",
	"vertical_speed": 1,
	"relocation_line": WINDOW_DIMENSIONS[1],
	"recreation_area": ( 0, -100, 640, 0 ),
	"creation_area": ( 0, 0, 640, 480 ),
	"interval": 5,
	"interval_stepping": 1	
}


# Enemy A
ENEMY_A = {
	"images": ( "enemy_a_1.png", "enemy_b_2.png" ),
	"transparency_colorkey": WHITE,
	"center_xy_position": [ 320, 240 ], # To be set dynamically.
	"interval": 25,
	"interval_stepping": 1,
	"removal_area": pygame.Rect( -1000, 480, 2640, 100 ),
	"speed": 5,
	"projectile_data": {
		"filename": "projectile_enemy.png",
		"transparency_colorkey": WHITE,
		"speed": 10, # Dynamically set.
		"center_xy_position": [ 0, 0 ], # Dynamically set.
		"angle": 0, # Will be determined during the game.
		"visible_area": ( 0, 0, 640, 480 ),
		"interval": 1,
		"interval_stepping": 1
	}
}
# Enemy B
ENEMY_B = {
	"images": ( "enemy_b_1.png", "enemy_b_2.png" ),
	"transparency_colorkey": WHITE,
	"center_xy_position": [ 320, 280 ], # To be set dynamically.
	"interval": 25,
	"interval_stepping": 1,
	"removal_area": pygame.Rect( -1000, 480, 2640, 100 ),
	"speed": 5,
	"projectile_data": {
		"filename": "projectile_enemy.png",
		"transparency_colorkey": WHITE,
		"speed": 10, # Dynamically set.
		"center_xy_position": [ 0, 0 ], # Dynamically set.
		"angle": 0, # Will be determined during the game.
		"visible_area": ( 0, 0, 640, 480 ),
		"interval": 1,
		"interval_stepping": 1
	}	
}
# Enemy C
ENEMY_C = {
	"images": ( "enemy_c_1.png", "enemy_c_2.png" ),
	"transparency_colorkey": WHITE,
	"center_xy_position": [ 120, 240 ], # To be set dynamically.
	"interval": 25,
	"interval_stepping": 1,
	"removal_area": pygame.Rect( -1000, 480, 2640, 100 ),
	"speed": 5,
	"projectile_data": {
		"filename": "projectile_enemy.png",
		"transparency_colorkey": WHITE,
		"speed": 10, # Dynamically set.
		"center_xy_position": [ 0, 0 ], # Dynamically set.
		"angle": 0, # Will be determined during the game.
		"visible_area": ( 0, 0, 640, 480 ),
		"interval": 1,
		"interval_stepping": 1
	}	
}
# Enemy D
ENEMY_D = {
	"images": ( "enemy_d_1.png", "enemy_d_2.png" ),
	"transparency_colorkey": WHITE,
	"center_xy_position": [ 220, 240 ], # To be set dynamically.
	"interval": 25,
	"interval_stepping": 1,
	"removal_area": pygame.Rect( -1000, 480, 2640, 100 ),
	"speed": 5,	# Will be set dynamically.
	"projectile_data": {
		"filename": "projectile_enemy.png",
		"transparency_colorkey": WHITE,
		"speed": 10, # Dynamically set.
		"center_xy_position": [ 0, 0 ], # Dynamically set.
		"angle": 0, # Will be determined during the game.
		"visible_area": ( 0, 0, 640, 480 ),
		"interval": 1,
		"interval_stepping": 1
	}	
}

ENEMY_MOVEMENTS = ( "weave", "slant_right", "slant_left", "circular", "normal" )


# BITS
# BitOn
BIT_ON = {
	"filename": "bit_on.png",
	"center_xy_position": [ 0, 0 ], # To be set dynamically.
	"transparency_colorkey": WHITE,
	"speed": 5,
	"gravity_amount": 0.5,
	"angle": 0, # To be set dynamically.
	"window_dimensions": ( 0, -200, 640, 680 ),
	"time_addition": 1 / FRAMERATE
}
# BitOff
BIT_OFF = {
	"filename": "bit_off.png",
	"center_xy_position": [ 0, 0 ], # To be set dynamically.
	"transparency_colorkey": WHITE,
	"speed": 5,
	"gravity_amount": 0.01,
	"angle": 0, # To be set dynamically.
	"window_dimensions": ( 0, -200, 640, 680 ),
	"time_addition": 1 / FRAMERATE	
}

# BitWalker
BIT_WALKER = {
	"bit_positions": (
		( 1, ( 462 + 12, 12 ) ),
		( 2, ( 484 + 12, 12 ) ),
		( 3, ( 506 + 12, 12 ) ),
		( 4, ( 528 + 12, 12 ) ),
		( 5, ( 550 + 12, 12 ) ),
		( 6, ( 572 + 12, 12 ) ),
		( 7, ( 594 + 12, 12 ) ),
		( 8, ( 616 + 12, 12 ) )
	),
	"filename": "bit_walker.png",
	"changetime": FRAMERATE * 2,
	"chosen_bit": 0,
	"transparency_colorkey": BLACK
}


# EightBit
EIGHT_BIT = {
	"filename": "eight_bits_table.png",
	"position_xy_center": ( 640 - 178 + 89, 12 ),
	"bit_walker_data": BIT_WALKER,
	"image_bit_on": "bit_on.png",
	"image_bit_off": "bit_off.png",
	"image_no_bit": "no_bit.png",
	"transparency_colorkey": WHITE,
	"bit_drawing_center_xy_positions": (
		( 462 + 12, 12 ),
		( 484 + 12, 12 ),
		( 506 + 12, 12 ),
		( 528 + 12, 12 ),
		( 550 + 12, 12 ),
		( 572 + 12, 12 ),
		( 594 + 12, 12 ),
		( 616 + 12, 12 )
	),
	"byte_assemblies": {
			"00000001": ( "score_bonus", 1 ),
			"00000011": ( "score_bonus", 3 ),
			"00000111": ( "score_bonus", 7 ),
			"00001111": ( "score_bonus", 15 ),
			"00011111": ( "score_bonus", 31 ),
			"00111111": ( "score_bonus", 63 ),
			"01111111": ( "score_bonus", 127 ),
			"11111111": ( "score_bonus", 255 ),
			"10101010": ( "weapon_upgrade", 1 ),
			"01010101": ( "increase_health", 10 )
	}
}

GFORCE = 9.81


# Text
CHARACTERS = {
	"A": """\
#######
#     #
#     #
#######
#     #
#     #
#     #""",
	"a": """\


#######
#     #
#######
#     #
#     #""",
	"B": """\
###### 
#     #
#     #
######
#     #
#     #
######""",
	"b": """\


###### 
#     #
######
#     #
######""",
	"C": """\
#######
#
#
#
#
#
#######""",
	"D": """\
######
#     #
#     #
#     #
#     #
#     #
######""",
	"E": """\
#######
#
#
#######
#
#
#######""",
	"F": """\
#######
#
#
####
#
#
#""",
	"G": """\
#######
#
#
# #####
# #   #
#     #
#######""",
	"H": """\
#     #
#     #
#     #
#######
#     #
#     #
#     #""",
	"I":"""\
#######
   #
   #
   #
   #
   #
#######""",
	"i":"""\


#######
   #
   #
   #
#######""",
	"J":"""\
#######
      #
      #
      #
      #
      #
#######""",
	"j":"""\


#######
      #
      #
      #
#######""",
	"K": """\
#   ###
# ##
##
##
# ##
#  ##
#   ###""",
	"k": """\


#   ###
# ##
##
# ##
#   ###""",
	"L": """\
#
#
#
#
#
#
#######""",
	"M": """\
#     #
##   ##
# # # #
# # # #
#  #  #
#     #
#     #""",
	"N": """\
#     #
##    #
# #   #
#  #  #
#   # #
#    ##
#     #""",
	"n": """\


##    #
# #   #
#  #  #
#   # #
#    ##""",
	"O": """\
#######
#     #
#     #
#     #
#     #
#     #
#######""",
	"P": """\
#######
#     #
#     #
#######
#
#
#""",
	"Q": """\
#######
#     #
#     #
#     #
#    ## 
#   ## 
##### #""",
	"R": """\
#######
#     #
#     #
#######
# #
#  ##
#   ###""",
	"S": """\
#######
#
#
#######
      #
      #
#######""",
	"T": """\
#######
   #
   #
   #
   #
   #
   #""",
   	"U": """\
#     #
#     #
#     #
#     #
#     #
#     #
#######""",
	"V": """\
#     #
#     #
 #   #
 #   #
  # #
   #
   #""",
   	"W": """\
#     #
#     #
#     #
#  #  #
#  #  #
# # # #
##   ##""",
	"X": """\
#     #
 #   #
  # #
   #
  # #
 #   #
#     #""",
	"Y": """\
#     #
 #   #
  # #
   #
   #
   #
   #""",
   	"Z": """\
#######
     #
    #
   #
  #
 #
#######""",
	".": """\






#""",
	" ": """\







""",
	"0": """\
#######
#     #
#     #
#     #
#     #
#     #
#######""",
	"1": """\
####
   #
   #
   #
   #
   #
#######""",
	"2": """\
 #####
#     # 
     #
    #
   #
 ##
#######""",
	"3": """\
#######
      #
      #
   ####
      #
      #
#######""",
	"4": """\
#
#
#
#  #
#  #
#######
   #""",
	"5": """\
#######
#
#######
      #
      #
      #
#######""",
	"6": """\
#######
#
#
#######
#     #
#     #
#######""",
	"7": """\
#######
     #
    #
   #
  #
 #
#""",
	"8": """\
#######
#     #
#     #
#######
#     #
#     #
#######""",
	"9": """\
#######
#     #
#     #
#######
      #
      #
#######""",
	"%": """\
###   #
# #  # 
### #
   #
  # ###
 #  # #
#   ###""",
	"-": """\



#######



""",
	":": """\

#



#

""",
	"!": """\
#
#
#
#
#

#""",
	"?": """\
 #####
#     #
     #
    #
   #

   #""",
	"'": """\
   #
   #




""",
	",": """\




##
##
#
"""
}


# Enemy waves
wave_number = 0

# Dead time.
DEAD_TIME = FRAMERATE * 2


# MENUES
MAIN_MENU = {
	"heading": {
		"sentence": "DEEP SPACE",
		"top": 50,
		"left": 50
	},
	"options": {
		0: {
			"sentence_data": {
				"sentence": "PLAY!",
				"top": 80,
				"left": 50			
			},
			"action": "game"
		},
		1: {
			"sentence_data": {
				"sentence": "HELP",
				"top": 90,
				"left": 50			
			},
			"action": "help"
		},
		2: {
			"sentence_data": {
				"sentence": "HIGHSCORE",
				"top": 100,
				"left": 50
			},
			"action": "highscore"
		},
		3: {
			"sentence_data": {
				"sentence": "EXIT",
				"top": 110,
				"left": 50
			},
			"action": "exit"
		}
	},
	"chosen_option": 0,
	"chosen_option_color": YELLOW
}

HELP_MENU = {
	"heading": {
		"sentence": "HELP",
		"top": 50,
		"left": 50
	},
	"options": {
		0: {
			"sentence_data": {
				"sentence": "BACK",
				"top": 80,
				"left": 50
			},
			"action": "exit"
		}
	},
	"chosen_option": 0,
	"chosen_option_color": YELLOW
}

HIGHSCORE_MENU = {
	"heading": {
		"sentence": "HIGHSCORE",
		"top": 50,
		"left": 50
	},
	"options": {
		0: {
			"sentence_data": {
				"sentence": "BACK",
				"top": 80,
				"left": 50
			},
			"action": "exit"
		}
	},
	"chosen_option": 0,
	"chosen_option_color": YELLOW
}

# Highscore file.
HIGHSCORE_FILE = os.path.join( os.getcwd(), "highscore", "highscore" )
HIGHSCORE_SENTENCE_POSITION_TOP = 50
HIGHSCORE_SENTENCE_POSITION_LEFT = 150
MAX_HIGHSCORE_ENTRIES = 25

# Name entering routine
ENTER_NAME_MESSAGE = "PLEASE ENTER YOUR NAME AND HIT ENTER:"
ENTER_NAME_MESSAGE_TOP = 50
ENTER_NAME_MESSAGE_LEFT = 50

PLAYER_NAME_TOP = 90
PLAYER_NAME_LEFT = 50
