#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()

import random
import copy

import code.functions.particles as particle_functions
from code.classes.BitOn import BitOn
from code.classes.BitOff import BitOff
import code.constants.constants as constants


def collision_bits_player( bits, player, eightbit ):

	collided_sprites = pygame.sprite.spritecollide( player.sprite, bits, True )
	for sp in collided_sprites:
		eightbit.sprite.set_bit( sp.status )
		eightbit.sprite.check_byte( player.sprite )


def collision_enemies_playershots( player, enemies ):

	sprites = pygame.sprite.groupcollide( enemies, player.sprite.projectiles,
		True, True )


	# Create the particles.
	particles = pygame.sprite.Group()

	# Create bits.
	bits = pygame.sprite.Group()
	
	for enemy_sprite in sprites:
		particles = particle_functions.surface_to_particles( enemy_sprite.image,
			enemy_sprite.rect.top, enemy_sprite.rect.left,
			enemy_sprite.transparency_colorkey,
			enemy_sprite.speed_x,
			enemy_sprite.speed_y
		)

		# Update the score.
		player.sprite.score += 1
		player.sprite.update_score()

		# Maybe create a bit where the enemy was.
		if random.randint( 1, 1 ) == 1:
		
			if random.randint( 1, 2 ) == 1:
				# Define the center of the bit.
				bit_data = copy.deepcopy( constants.BIT_ON )
				bit_data["center_xy_position"] = enemy_sprite.rect.center
				
				# Define the direction angle.
				bit_data["angle"] = random.randint( 240, 300 )
				bits.add( BitOn( **bit_data ) )
				
			else:
				# Define the center of the bit.
				bit_data = copy.deepcopy( constants.BIT_OFF )
				bit_data["center_xy_position"] = enemy_sprite.rect.center
				
				# Define the direction angle.
				bit_data["angle"] = random.randint( 240, 300 )
				bits.add( BitOff( **bit_data ) )
		
	return particles, bits


def collision_player_enemyshots( player, enemy_projectiles ):

	collided_sprites = pygame.sprite.spritecollide( player.sprite,
		enemy_projectiles, True )


	if len( collided_sprites ) > 0:

		for x in collided_sprites:
			player.sprite.health -= 1
			player.sprite.update_health()
			
		p = player.sprite

		if p.health <= 0:
			p.rect.center = ( -100, -100 )
			p.dead = True
		
		# Create the particles.
		particles = particle_functions.surface_to_particles( p.image,
			p.rect.top, p.rect.left,
			p.transparency_colorkey
		)

				
		return particles
		
	else:
		return None


def collision_enemies_player( enemies, player ):

	collided_sprites = pygame.sprite.spritecollide( player.sprite, enemies, True )
	if len( collided_sprites ) > 0:

		p = player.sprite
		
		# Create the particles for the player.
		particles = particle_functions.surface_to_particles( p.image,
			p.rect.top, p.rect.left,
			p.transparency_colorkey
		)

		# Create the particles for the enemy objects.
		for enemy_sprite in collided_sprites:
			particles.add( particle_functions.surface_to_particles( enemy_sprite.image,
				enemy_sprite.rect.top, enemy_sprite.rect.left,
				enemy_sprite.transparency_colorkey,
				enemy_sprite.speed_x,
				enemy_sprite.speed_y )
			)			

		p.health = 0
		p.update_health()
		p.rect.center = ( -100, -100 )
		p.dead = True
				
		return particles

	return None
		
