import pygame
pygame.init()

import os

from code.classes.PlayerSpaceship import PlayerSpaceship
from code.classes.Star import Star
from code.classes.EnemySpaceship import EnemySpaceship
from code.classes.EightBit import EightBit
from code.classes.Sentence import Sentence
from code.classes.Menu import Menu


import code.constants.constants as constants
import code.functions.collision as collision
import code.functions.waves as waves
import code.functions.highscore as highscore_functions


import copy
import random


def main():
	main_menu()


def main_menu():
	window = pygame.display.set_mode( constants.WINDOW_DIMENSIONS )
	clock = pygame.time.Clock()
	pygame.display.set_caption( constants.GAME_TITLE )

	menu = Menu( **constants.MAIN_MENU )

	running = True
	while running:
		events = pygame.event.get()
		for e in events:
			if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
				running = False

		window.fill( constants.BLACK )
		result = menu.update( events )
		if result is not None:
			if result == "exit":
				running = False
			else:
				globals()[result]( window, clock)
			
		menu.draw( window )

		pygame.display.flip()

		clock.tick( constants.FRAMERATE )

	pygame.display.quit()
		

def highscore( window, clock ):

	# Check if a highscore file exists.
	if not os.path.exists( constants.HIGHSCORE_FILE ):
		return
		
	highscore_list = highscore_functions.read_highscore()
	
	highscore_sentence = highscore_functions.render_highscore( highscore_list,
		constants.HIGHSCORE_SENTENCE_POSITION_TOP,
		constants.HIGHSCORE_SENTENCE_POSITION_LEFT )
	
	menu = Menu( **constants.HIGHSCORE_MENU )

	running = True
	while running:
		events = pygame.event.get()
		for e in events:
			if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
				running = False

		window.fill( constants.BLACK )
		result = menu.update( events )
		if result is not None:
			if result == "exit":
				running = False
			else:
				globals()[result]( window, clock)
			
		menu.draw( window )
		highscore_sentence.draw( window )

		clock.tick( constants.FRAMERATE )		

		pygame.display.flip()


def help( window, clock ):

	menu = Menu( **constants.HELP_MENU )

	helptext = Sentence(
"""THE OBJECTIVE

THE AIM IS TO GET A REALLY HIGH SCORE.  THIS CAN BE
ACHIEVED BY SHOOTING ENEMIES AND COLLECTING THEIR
DROPPED BIT'S TO BUILD VARIOUS BYTE COMBINATIONS,
WHICH CAN FURTHER IMPROVE THE HIGH SCORE.  SHOOTING
AN ENEMY WILL GET ONE 1 POINT.

ALIENS ARE CONSIDERED TO BE THE FOUR COLOURED
RECTANGLES.

TO BUILD A BYTE, SIMPLY COLLECT THE DROPPED 1'S AND
0'S.  THE BYTE-RUNNER WILL TAKE CARE OF ASSIGNING
THE BITS TO THEIR DEFINITE POSITIONS.  IT MOVES TO
THE NEXT BIT EVERY 2 SECONDS.  THE POSITION OF THE
BYTE-RUNNER IS INDICATED BY THE WHITE RECTANGLE.


HOW TO CONTROL THE SPACESHIP

LEFT ARROW - MOVE LEFT
RIGHT ARROW - MOVE RIGHT
UP ARROW - MOVE UP
DOWN ARROW - MOVE DOWN
RIGHT CTRL - FIRE


BYTE COMBINATIONS

SCORE:

0000 0001 -   1
0000 0011 -   3
0000 0111 -   7
0000 1111 -  15
0001 1111 -  31
0011 1111 -  63
0111 1111 - 127
1111 1111 - 255

EXTRAS:

1010 1010 - WEAPSON UPGRADE
0101 0101 - INCREASE HEALTH BY 10 %
""", 50, 150 )

	running = True
	while running:
		events = pygame.event.get()
		for e in events:
			if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
				running = False

		window.fill( constants.BLACK )
		result = menu.update( events )
		if result is not None:
			if result == "exit":
				running = False
			else:
				globals()[result]( window, clock)
			
		menu.draw( window )

		helptext.draw( window )

		clock.tick( constants.FRAMERATE )		

		pygame.display.flip()


def enter_highscore( window, clock, score ):
	pass


def game( window, clock ):

	events = None

	# Create 25 stars.
	stars = pygame.sprite.Group()
	for x in range( 25 ):
		stars.add( Star( **constants.STAR ) )

	# Create the player spaceship.
	ps = pygame.sprite.GroupSingle( PlayerSpaceship( **copy.deepcopy( constants.PLAYER1 ) ) )

	# Let's create enemies!
	enemies = pygame.sprite.Group()
	enemy_projectiles = pygame.sprite.Group()
	
	wave_number = constants.wave_number

	# Let's create a group that holds the particles of the destroyed objects.
	particles = pygame.sprite.RenderUpdates()			

	# Create a group that holds any bits.
	bits = pygame.sprite.Group()


	# Create the 8-bit board.
	eightbit = pygame.sprite.GroupSingle( EightBit( **constants.EIGHT_BIT ) )


	# Display the current wave in the top-right corner.
	wave_display = Sentence( "WAVE: " + str( wave_number ), 60, 462 )

	# Game over message.
	game_over_message = Sentence( "GAME OVER!", constants.WINDOW_DIMENSIONS[1] / 2,
		( constants.WINDOW_DIMENSIONS[0] / 2 ) - 30 )
	


	dead_timer = 0
	
	running = True
	while running:
		events = pygame.event.get()
		for e in events:
			if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
				running = False

		window.fill( constants.BLACK )

		# Update the stars.
		stars.update()

		# Update the particles.
		particles.update()

		# Update the bits.
		bits.update()

		# Update the enemies.
		enemies.update( ps, enemy_projectiles, wave_number )
		enemy_projectiles.update()

		# Update the spaceship of player #1.
		ps.sprite.update( events )

		if ps.sprite.dead:
			if dead_timer < constants.DEAD_TIME:
				dead_timer += 1
			else:
				running = False

		# Update the shots fired by player #1.
		ps.sprite.projectiles.update()

		# Check for any collisions between the player and the enemies.
		result, result_bits = collision.collision_enemies_playershots( ps, enemies )
		if len( result ) > 0:
			particles.add( result )
			bits.add( result_bits )
			
		result = collision.collision_player_enemyshots( ps, enemy_projectiles )
		if result != None:
			particles.add( result )

		collision.collision_bits_player( bits, ps, eightbit )

		result = collision.collision_enemies_player( enemies, ps )
		if result is not None:
			particles.add( result )

		# Update the eight bits table.
		eightbit.update()

		# Check if the next wave should be created.
		if waves.wave_completed( enemies ):
			wave_number, enemies = waves.next_wave( enemies, wave_number )
			wave_display = Sentence( "WAVE: " + str( wave_number ), 60, 462 )
		
		


		# Draw the stars.
		stars.draw( window )

		# Draw the particles.
		particles_rectangeles = particles.draw( window )

		# Draw the bits.
		bits.draw( window )

		# Draw the enemies.
		enemies.draw( window )

		enemy_projectiles.draw( window )

		# Draw the shots fired by player #1.
		ps.sprite.projectiles.draw( window )

		# Draw the spaceship of Player #1.
		ps.draw( window )

		# Draw the eight bits table.
		eightbit.draw( window )
		eightbit.sprite.bits_to_draw.draw( window )

		# Draw the bitwalker.
		eightbit.sprite.bitwalker.draw( window )

		# Display the score.
		ps.sprite.score_display.draw( window )

		# Display the health.
		ps.sprite.health_display.draw( window )

		# Display the projectile level.
		ps.sprite.projectile_level_display.draw( window )

		# Display the wave.
		wave_display.draw( window )

		# Display the game over message if the player has died.
		if ps.sprite.dead:
			game_over_message.draw( window )

		pygame.display.update()

		clock.tick( constants.FRAMERATE )


	# The game is over.

	# First let the player enter his/her name.
	player_name = enter_name_routine( window, clock )

	# First check if a highscore table exists.  If not, then create it.
	if not os.path.exists( constants.HIGHSCORE_FILE ):
		highscore_functions.create_highscore()

		# Write the current highscore to the file and close it.
		highscore_list = list()
		highscore_list.append( [ player_name, ps.sprite.score ] )

		highscore_functions.write_highscore( highscore_list )

	else:
		# Read the entire highscore list.
		highscore_list = highscore_functions.read_highscore()
		highscore_list.append( [ player_name, ps.sprite.score ] )

		highscore_list = highscore_functions.sort_highscore( highscore_list )
		highscore_list = highscore_list[0:constants.MAX_HIGHSCORE_ENTRIES]

		highscore_functions.write_highscore( highscore_list )


def enter_name_routine( window, clock ):

	enter_name_message = Sentence( constants.ENTER_NAME_MESSAGE,
		constants.ENTER_NAME_MESSAGE_TOP,
		constants.ENTER_NAME_MESSAGE_LEFT
	)

	player_name = ""
	tmp_player_name = ""

	display_player_name = Sentence( player_name, constants.PLAYER_NAME_TOP,
				constants.PLAYER_NAME_LEFT )

	running = True
	while running:
	
		events = pygame.event.get()
		for e in events:
			if e.type == pygame.KEYDOWN:

				if e.key == pygame.K_RETURN:
					running = False
					break

				if e.key == 8 and len( player_name ) >= 1:
					player_name = player_name[:-1]
				elif e.key >= 97 and e.key <= 122:
					player_name += chr( e.key ).upper()
				elif e.key >= 48 and e.key <= 57:
					player_name += chr( e.key )
				else:
					continue

		if player_name != tmp_player_name:
			display_player_name = Sentence( player_name, constants.PLAYER_NAME_TOP,
				constants.PLAYER_NAME_LEFT )
			
		tmp_player_name = player_name

		window.fill( constants.BLACK )
		enter_name_message.draw( window )
		display_player_name.draw( window )
		
		clock.tick( constants.FRAMERATE )

		pygame.display.flip()

	if player_name == "":
		player_name = "UNKNOWN"

	return player_name
main()
